''  jm_con.bas
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

#include "jm_con.bi"
#include "con_buf.bi"
#include "crt/stdio.bi"
#include "crt/string.bi"
#include "../jm_hack/jm_hack.bi"

'' Allocate some space for the console
#define DEFMAXROW   20
#define DEFMAXCOL   40
#define DEFCONSIZE  DEFMAXROW * DEFMAXCOL

#define TRUE  1
#define FALSE 0

Dim Shared defcon As CONBUFFER_T Ptr

Dim Shared conechoflag As Integer = FALSE  '' by default, do not echo to system console
Dim Shared condebugflag As Integer = FALSE '' by default, do not print debug messages

Dim shared logfile As zString * 260
Dim shared logenabled as integer

''BEGIN_CTOR(jm_con_init)
private sub jm_con_init alias "jm_con_init" () FB_CONSTRUCTOR
''Private sub jm_con_init () Constructor
	dim defcondata as UByte Ptr
	defcon = Allocate(sizeof(CONBUFFER_T))
	if defcon Then
		defcondata = Allocate(DEFCONSIZE)
		defcon->maxrow  = DEFMAXROW
		defcon->maxcol  = DEFMAXCOL
		defcon->row     = 0
		defcon->col     = 0
		defcon->size    = DEFCONSIZE
		defcon->changed = TRUE
		defcon->dataptr = defcondata
		CON_clear
	End If
	logfile[0] = 0
	logenabled = FALSE
	EMIT_GLOBAL_CTOR(jm_con_init)
END_CTOR

Private Sub jm_con_exit cdecl alias "jm_con_exit" () FB_DESTRUCTOR
''BEGIN_DTOR(jm_con_exit)
''Private sub jm_con_exit () Destructor
	If (defcon) Then
		If (defcon->dataptr) Then
			Deallocate defcon->dataptr
		End If
		Deallocate defcon
	End If
	EMIT_GLOBAL_DTOR(jm_con_exit)
END_DTOR

public Function CON_getecho alias "CON_getecho" (  ) As Integer
	return(conechoflag)
End Function

public Function CON_setecho alias "CON_setecho" ( ByVal flag As Integer) As Integer
	Dim oldflag As Integer = conechoflag
	conechoflag = flag
	return(oldflag)
End Function

public Function CON_getdebug alias "CON_getdebug" (  ) As Integer
	return(condebugflag)
End Function

public Function CON_setdebug alias "CON_setdebug" ( ByVal flag As Integer ) As Integer
	Dim oldflag As Integer = condebugflag
	condebugflag = flag
	return(oldflag)
End Function

public Sub CON_setlogfile alias "CON_setlogfile" ( byval f As zString ptr)
	if (f) then
		strcpy(logfile, f)
		if logfile[0] then
			logenabled = TRUE
		else
			logenabled = FALSE
		endif
	endif
End Sub

public Function CON_getlogfile alias "CON_getlogfile" () As zString Ptr
	function = @logfile
End Function

public Sub CON_clear alias "CON_clear" ( )
	conbuf_clear defcon
End Sub

public Sub CON_print1 alias "CON_print" (byval txt As zString ptr)
	if (conechoflag) Then
		printf("%s", txt)
	End If

	conbuf_text defcon, txt

	If logenabled then
		dim h as FILE ptr
		h = fopen(logfile, "at")
		if (h) then
			fputs(txt, h)
			fclose(h)
		endif
	End If
End Sub

public Sub CON_print1_dbg alias "CON_print_dbg" (byval txt As zstring ptr)
	if (condebugflag <> 0) Then
		CON_print1 txt
	End If
End Sub

public sub CON_print alias "CON_print" (byval txt As zString ptr)
	CON_print1 txt
	CON_print1 chr(13) & chr(10)
end sub

public sub CON_print_dbg alias "CON_print_dbg" (byval txt As zstring ptr)
	if (condebugflag <> 0) Then
		CON_print1 txt
		CON_print1 chr(13) & chr(10)
	End If
end sub

public Function CON_data alias "CON_data" ( ) As UByte Ptr
	if (defcon) Then
		defcon->changed = FALSE
		return( defcon->dataptr )
	End If
End Function

public Function CON_maxrow alias "CON_maxrow" ( ) As Integer
	return( defcon->maxrow )
End Function

public Function CON_maxcol alias "CON_maxcol" ( ) As Integer
	return( defcon->maxcol )
End Function

public Function CON_resize alias "CON_resize" (ByVal cols As Integer, ByVal Rows As Integer) As Integer
	Dim tmpcondata As UByte ptr
	Dim tmpcon As CONBUFFER_T Ptr 
	
	If (cols > 0) And (cols <= 80) And (rows > 0) And (rows <= 50) Then
		tmpcon = Allocate(sizeof(CONBUFFER_T))
		if (tmpcon) Then
			tmpcondata = Allocate(DEFCONSIZE)
			if (tmpcondata) Then
				tmpcon->maxrow  = rows
				tmpcon->maxcol  = cols
				tmpcon->row     = 0
				tmpcon->col     = 0
				tmpcon->size    = rows * cols
				tmpcon->changed = TRUE
				tmpcon->dataptr = tmpcondata
				'' FIXME: preserve data between resize (copy)
				If (defcon) Then
					If (defcon->dataptr) Then
						Deallocate defcon->dataptr
					End If
					Deallocate defcon
				End If
				defcon = tmpcon
				CON_clear
				return 1
			Else
				Deallocate tmpcon
			End If
		End If
	End If
	return 0
End Function
