#ifndef __INCLUDE_JM_BOXES_BI__
#define __INCLUDE_JM_BOXES_BI__

#include "jm_gui.bi"
#include "jm_boxes_color.bi"

'' -------------------------------------------------------------------
'' UI ELEMENT TYPES
'' -------------------------------------------------------------------
        
Enum UI_BOX_TYPES
  UI_BOX_EMPTY = 0
  UI_BOX_ROOT
  UI_BOX_FRAME
  UI_BOX_CAPTION
  UI_BOX_BUTTON
	UI_BOX_CHECK
  UI_BOX_VSCROLL
	UI_BOX_INPUT
	UI_BOX_EDIT
End Enum

'' -------------------------------------------------------------------
'' EXTRA DATA (FOR SOME UI_BOX_TYPES)
'' -------------------------------------------------------------------

Type UI_CHECK_T
	checked as Integer
End Type

'' -------------------------------------------------------------------
'' BOX CREATION
'' -------------------------------------------------------------------

Declare Function UI_CreateRoot alias "UI_CreateRoot" (byval id As integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer ) As UI_BOX_PTR

Declare Function UI_CreateFrame alias "UI_CreateFrame" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
  byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer ) As UI_BOX_PTR

Declare Function UI_CreateCaption alias "UI_CreateCaption" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
  byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR

Declare Function UI_CreateButton alias "UI_CreateButton" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
  byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR

Declare Function UI_CreateCheck alias "UI_CreateCheck" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
  byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
  ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR

'' -------------------------------------------------------------------
'' DEFAULT BOXPROCS
'' -------------------------------------------------------------------

Declare Function UI_DefRootProc_cb alias "UI_DefRootProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
Declare Function UI_DefFrameProc_cb alias "UI_DefFrameProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
Declare Function UI_DefCaptionProc_cb alias "UI_DefCaptionProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
Declare Function UI_DefButtonProc_cb alias "UI_DefButtonProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
Declare Function UI_DefCheckProc_cb alias "UI_DefCheckProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer

'' -------------------------------------------------------------------
'' FRAMESTYLE CONSTANTS
'' -------------------------------------------------------------------

Const UI_FRAMESTYLE_NONE    = 0
Const UI_FRAMESTYLE_UP      = 1
Const UI_FRAMESTYLE_DOWN    = 2
Const UI_FRAMESTYLE_EDGE    = 3
Const UI_FRAMESTYLE_THIN    = 4
Const UI_FRAMESTYLE_THICK   = 5

Const UI_FRAMESTYLE_FILL    = 8
Const UI_FRAMESTYLE_OUTSIDE = 16

Const UI_FS_BTN_UP = UI_FRAMESTYLE_FILL or UI_FRAMESTYLE_UP
Const UI_FS_BTN_DN = UI_FRAMESTYLE_FILL or UI_FRAMESTYLE_DOWN
Const UI_FS_CAP    = UI_FRAMESTYLE_FILL
Const UI_FS_EDIT   = UI_FRAMESTYLE_FILL or UI_FRAMESTYLE_THICK or UI_FRAMESTYLE_OUTSIDE

Const UI_TEXT_ALIGN_LEFT = 0
Const UI_TEXT_ALIGN_CENTER = 1
Const UI_TEXT_ALIGN_RIGHT = 2

Const UI_TEXT_ALIGN_TOP = 0
Const UI_TEXT_ALIGN_MIDDLE = 4
Const UI_TEXT_ALIGN_BOTTOM = 8

Declare Sub UI_DrawCheckBox alias "UI_DrawCheckBox" (ByVal r As UI_RECT_T Ptr, ByVal Checked As Integer, ByVal clr As Integer)
Declare Sub UI_DrawFrame alias "UI_DrawFrame"  (ByVal r As UI_RECT_T Ptr, ByVal iStyle as integer)

Declare Sub UI_SetCaptionById alias "UI_SetCaptionById" (ByVal owner As UI_BOX_PTR, ByVal id As Integer, byval txt As zString Ptr)

'' -------------------------------------------------------------------
'' DRAWING CODE
'' -------------------------------------------------------------------

Declare Sub UI_DrawStdCaption alias "UI_DrawStdCaption" (ByVal r As UI_RECT_T Ptr, ByVal flags As Integer, byval Caption As zString ptr)
Declare Sub UI_DrawStdButton alias "UI_DrawStdButton" (ByVal r As UI_RECT_T Ptr, ByVal flags As Integer, byval Caption As zString ptr)
Declare Sub UI_DrawStdCheck alias "UI_DrawStdCheck" (ByVal r As UI_RECT_T Ptr, ByVal flags As Integer, ByVal d As UI_CHECK_T Ptr, byval Caption As zString ptr)

Declare Sub UI_DrawCaret alias "UI_DrawCaret" (ByVal x as integer, byval y as integer, byval w as integer, byval h as integer, byval clr as integer)

Declare Sub UI_DrawText alias "UI_DrawText" (ByVal r As UI_RECT_T Ptr, byval Caption As zString ptr, ByVal fc As Integer, ByVal bc As Integer, ByVal align as integer)
Declare Sub UI_DrawTextN alias "UI_DrawTextN"   (ByVal r As UI_RECT_T Ptr, ByVal Caption As ubyte Ptr, ByVal nBytes as integer, ByVal fc As Integer, ByVal bc As Integer, ByVal align as integer)

#endif
