''  jm_gap_buffer.bas - jm_gui
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General License for more details.
''
''	You should have received a copy of the GNU General License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

#include "jm_gap_buffer.bi"

Const MIN_GAP_SIZE = 32
Const False = 0, True = Not False

function GAP_BUFFER_Create alias "GAP_BUFFER_Create"  (byval startsize as integer) as GAP_BUFFER_T Ptr
	dim buf as GAP_BUFFER_T Ptr
	buf = Allocate(sizeof(GAP_BUFFER_T))
	if (buf) then
		if startsize < MIN_GAP_SIZE then
			buf->msize = MIN_GAP_SIZE
		else
			buf->msize = (startsize + 3) and (not 3)
		end if
		buf->mdata = Allocate(buf->msize)
		if (buf->mdata) then
			buf->mgap1 = 0
			buf->mgap2 = buf->msize
			buf->mindex = 0
			buf->mdata[0] = 0
		else
			Deallocate buf
			buf = 0
		end if
	end if
	function = buf
End Function

sub GAP_BUFFER_Destroy alias "GAP_BUFFER_Destroy" (byval buf as GAP_BUFFER_T Ptr)
	if (buf) then
		if buf->mdata then
			Deallocate buf->mdata
		end if
		Deallocate buf
	end if
End Sub

sub GAP_BUFFER_Clear alias "GAP_BUFFER_Clear" (byval buf as GAP_BUFFER_T Ptr)
	dim p as any ptr
	buf->msize = MIN_GAP_SIZE
	p = Reallocate(buf->mdata, buf->msize)
	if (p) then
		buf->mdata = p
	end if
	buf->mGap1 = 0
	buf->mGap2 = buf->msize
	buf->mIndex = 0
	buf->mflags or= GAP_BUFFER_FLAG_CHANGED
End Sub

Private sub MoveCharacters alias "MoveCharacters" (byval buf as GAP_BUFFER_T Ptr, byval dst As integer, byval src As integer, byval count As integer)
	'' FIXME: use crt::memmove()
	If count > 0 Then
		'' memmove buf->mdata + dst, buf->mdata + src, count
		Dim i as integer
		if dst > src then
			for i = count - 1 to 0 step - 1
				buf->mdata[dst + i] = buf->mdata[src + i]
			next i
		else
			for i = 0 to count - 1
				buf->mdata[dst + i] = buf->mdata[src + i]
			next i
		end if
	End If
End Sub

Private sub MoveGapToPointer alias "MoveGapToPointer" (byval buf as GAP_BUFFER_T ptr)
	If buf->mindex = buf->mGap1 Then
		Exit Sub
	End If

	If buf->mindex = buf->mGap2 Then
		buf->mindex = buf->mGap1
		Exit Sub
	End If

	If buf->mindex < buf->mGap1 Then
		' move bytes between buf->mindex and buf->mGap1 to just before end of current gap
		MoveCharacters buf, buf->mindex + (buf->mGap2 - buf->mGap1), buf->mindex, buf->mGap1 - buf->mindex
		buf->mGap2 = buf->mGap2 - (buf->mGap1 - buf->mindex)
		buf->mGap1 = buf->mindex
	Else
		' move bytes between buf->mGap2 and buf->mindex to start of current gap
		MoveCharacters buf, buf->mGap1, buf->mGap2, buf->mindex - buf->mGap2
		buf->mGap1 = buf->mGap1 + (buf->mindex - buf->mGap2)
		buf->mGap2 = buf->mindex
		buf->mindex = buf->mGap1
	End If
End Sub

Private function GetGapSize alias "GetGapSize" (byval buf as GAP_BUFFER_T Ptr) As integer
	function = (buf->mGap2 - buf->mGap1)
End function
Private sub SetGapSize alias "SetGapSize" (byval buf as GAP_BUFFER_T Ptr, byval size As integer)
  ' FIXME: Allow public, adjust buf->mIndex after Gap Resize
	If size >= 0 Then
		If size > (buf->mGap2 - buf->mGap1) Then
			buf->mdata = Reallocate(buf->mdata, buf->msize - (buf->mGap2 - buf->mGap1) + size - 1)
			'' FIXME: check for failure
			MoveCharacters buf, buf->mGap1 + size, buf->mGap2, buf->msize - buf->mGap2
		ElseIf size < (buf->mGap2 - buf->mGap1) Then
			MoveCharacters buf, buf->mGap1 + size, buf->mGap2, buf->msize - buf->mGap2
			buf->mdata = Reallocate(buf->mdata, buf->msize - (buf->mGap2 - buf->mGap1) + size - 1)
			'' FIXME: check for failure
		End If
		buf->msize = buf->msize - (buf->mGap2 - buf->mGap1) + size
		buf->mGap2 = buf->mGap1 + size
	End If
End sub

function GAP_BUFFER_GetBufferSize alias "GAP_BUFFER_GetBufferSize" (byval buf as GAP_BUFFER_T ptr) As integer
  function = buf->msize
End function
sub GAP_BUFFER_SetBufferSize alias "GAP_BUFFER_SetBufferSize" (byval buf as GAP_BUFFER_T ptr, byval size As integer)
	If size > buf->msize Then
		SetGapSize buf, (size - buf->msize) + (buf->mGap2 - buf->mGap1)
	Else
		' TODO: Allow resize smaller
	End If
End sub

' ===========================================================================
' CLASS FUNCTIONS
' ===========================================================================

Private function PointerToPtr alias "PointerToPtr" (byval buf as GAP_BUFFER_T ptr, byval Index As integer, byval bAfterGap As integer) As integer
  Dim iptr As integer
	If Index < 0 Then
		iptr = 0
	ElseIf Index > buf->msize - (buf->mGap2 - buf->mGap1) Then
		iptr = buf->msize
	ElseIf Index >= buf->mGap1 Then
		iptr = Index + (buf->mGap2 - buf->mGap1)
	Else
		iptr = Index
	End If
	If bAfterGap Then
		If iptr = buf->mGap1 Then
			iptr = buf->mGap2
		End If
	Else
		If iptr = buf->mGap2 Then
			iptr = buf->mGap1
		End If
	End If
	function = iptr
End Function

Private function PtrToPointer alias "PtrToPointer" (byval buf as GAP_BUFFER_T ptr, byval iptr As integer) As integer
	If iptr < 0 Then
		function = 0
	ElseIf iptr >= buf->msize Then
		function = buf->msize - (buf->mGap2 - buf->mGap1)
	ElseIf iptr >= buf->mGap2 Then
		function = iptr - (buf->mGap2 - buf->mGap1)
	ElseIf iptr > buf->mGap1 Then
		function = buf->mGap1
	Else
		function = iptr
	End If
End Function

' ===========================================================================
' METHODS / PROPERTIES
' ===========================================================================

function GAP_BUFFER_GetLength alias "GAP_BUFFER_GetLength" (byval buf as GAP_BUFFER_T ptr) As integer
  function = buf->msize - (buf->mGap2 - buf->mGap1)
End function
function GAP_BUFFER_GetIndex alias "GAP_BUFFER_GetIndex" (byval buf as GAP_BUFFER_T ptr) As integer
  function = PtrToPointer(buf, buf->mIndex)
End function
sub GAP_BUFFER_SetIndex alias "GAP_BUFFER_SetIndex" (byval buf as GAP_BUFFER_T ptr, byval Index As integer)
  buf->mIndex = PointerToPtr(buf, Index, False)
End sub

sub GAP_BUFFER_InsertChar alias "GAP_BUFFER_InsertChar" (byval buf as GAP_BUFFER_T Ptr, byval ch As Integer)
  MoveGapToPointer(buf)
  If GetGapSize(buf) < 1 Then
    SetGapSize(buf,MIN_GAP_SIZE)
  End If
  buf->mdata[buf->mIndex] = ch
  buf->mIndex = buf->mIndex + 1
  buf->mGap1 = buf->mGap1 + 1
	buf->mflags or= GAP_BUFFER_FLAG_CHANGED
End Sub

sub GAP_BUFFER_BackChar alias "GAP_BUFFER_BackChar" (byval buf as GAP_BUFFER_T Ptr)
  MoveGapToPointer(buf)
  If buf->mGap1 > 0 Then
    buf->mGap1 = buf->mGap1 - 1
    buf->mIndex = buf->mIndex - 1
		buf->mflags or= GAP_BUFFER_FLAG_CHANGED
  End If
End Sub

sub GAP_BUFFER_DeleteChar alias "GAP_BUFFER_DeleteChar" (byval buf as GAP_BUFFER_T Ptr)
  MoveGapToPointer(buf)
  If buf->msize - buf->mGap2 > 0 Then
    buf->mGap2 = buf->mGap2 + 1
		buf->mflags or= GAP_BUFFER_FLAG_CHANGED
  End If
End Sub

function GAP_BUFFER_GetChar alias "GAP_BUFFER_GetChar" (byval buf as GAP_BUFFER_T Ptr, byval Index As integer) As Integer
	function = -1
  If Index >= 0 And Index < GAP_BUFFER_GetLength(buf) Then
		dim i as integer = PointerToPtr(buf, Index, True)        '' VER_COMPAT: needed for 0.15 or lower
    function = buf->mdata[i]                                 '' VER_COMPAT: needed for 0.15 or lower
    '' function = buf->mdata[PointerToPtr(buf, Index, True)] '' VER_COMPAT: Ok for 0.16 or higher
  End If
End function
sub GAP_BUFFER_SetChar alias "GAP_BUFFER_SetChar" (byval buf as GAP_BUFFER_T Ptr, byval Index As integer, byval ch As Integer)
  If Index >= 0 And Index < GAP_BUFFER_GetLength(buf) Then
		dim i as integer = PointerToPtr(buf, Index, True)   '' VER_COMPAT: needed for 0.15 or lower
    buf->mdata[i] = ch																	'' VER_COMPAT: needed for 0.15 or lower
    '' buf->mdata[PointerToPtr(buf, Index, True)] = ch  '' VER_COMPAT: Ok for 0.16 or higher
  End If
End sub

function GAP_BUFFER_GetText alias "GAP_BUFFER_GetText" (byval buf as GAP_BUFFER_T Ptr, ByVal txt as ubyte ptr, ByVal size as integer) as integer
	dim i as integer
	function = 0
	if (buf) then
		if (txt <> 0) and (size > 0) then
			if buf->mGap1 > 0 then
				'' FIXME: use memcopy()
				for i = 0 to buf->mGap1 - 1
					txt[i] = buf->mdata[i]
				next i
			end if
			If buf->msize - buf->mGap2 > 0 Then
				'' FIXME: use memcopy()
				'' CopyMemory b(.mGap1), m(.mGap2), (.msize - .mGap2)
				for i = 0 to buf->msize - buf->mGap2
					txt[i + buf->mGap1] = buf->mdata[i + buf->mGap2]
				next i
			End If
			function = size
		else
			function = buf->msize - (buf->mGap2 - buf->mGap1)
		end if
	end if
End function

function GAP_BUFFER_SetText alias "GAP_BUFFER_SetText" (byval buf as GAP_BUFFER_T Ptr, ByVal txt as ubyte ptr, ByVal size as integer) as integer
	dim i as integer
	function = 0
	if (txt <> 0) and (size > 0) then
		if size > buf->msize then
			GAP_BUFFER_SetBufferSize(buf,size)
		end if
		'' FIXME: use memcopy()
		for i = 0 to size - 1
			buf->mdata[i] = txt[i]
		next i
		buf->mGap1 = size
		buf->mGap2 = buf->msize
		buf->mIndex = 0

		buf->mflags or= GAP_BUFFER_FLAG_CHANGED
		function = size
	else
		GAP_BUFFER_Clear(buf)
	end if
end function

sub GAP_BUFFER_ClearChanged alias "GAP_BUFFER_ClearChanged" (byval buf as GAP_BUFFER_T Ptr)
	buf->mflags and= (NOT GAP_BUFFER_FLAG_CHANGED)
end sub

function GAP_BUFFER_IsChanged alias "GAP_BUFFER_IsChanged" (byval buf as GAP_BUFFER_T Ptr) as integer
	function = (buf->mflags and GAP_BUFFER_FLAG_CHANGED)
	buf->mflags and= (NOT GAP_BUFFER_FLAG_CHANGED)
end function







#if 0

' ===========================================================================
' CHARACTER ROUTINES
' ===========================================================================

sub InsertCharacters alias "InsertCharacters" (byval buf as GAP_BUFFER_T Ptr, b() As Integer, n As integer)
  MoveGapToPointer
  If GapSize < n Then
    GapSize = n + MIN_GAP_SIZE
  End If
  MoveMemory m(.mPtr), b(0), n * 2
  .mPtr = .mPtr + n
  .mGap1 = .mGap1 + n
End Sub

sub InsertText alias "InsertText" (byval buf as GAP_BUFFER_T Ptr, byval t As zString ptr)
  Dim n As integer
  MoveGapToPointer
  n = Len(*t)
  If n > 0 Then
    If GapSize < n Then
      GapSize = n + MIN_GAP_SIZE
    End If
    CopyMemoryFromPtr m(.mPtr), StrPtr(t), n * 2
    .mPtr = .mPtr + n
    .mGap1 = .mGap1 + n
  End If
End Sub

sub DeleteCharacters alias "DeleteCharacters" (byval buf as GAP_BUFFER_T Ptr, StartIndex As integer, n As integer)
  If n > 0 Then
    .mPtr = PointerToPtr(StartIndex, False)
    MoveGapToPointer
    .mGap2 = PointerToPtr(StartIndex + n, True)
  End If
End Sub

sub DeleteToPointer alias "DeleteToPointer" (byval buf as GAP_BUFFER_T Ptr, StartIndex As integer)
  Dim n As integer, Index As integer
  Index = PtrToPointer(.mPtr)
  If Index < StartIndex Then
    DeleteCharacters Index, StartIndex - Index
  Else
    DeleteCharacters StartIndex, Index - StartIndex
  End If
End Sub

' ===========================================================================
' GET/SET PARTIAL CONTENTS
' ===========================================================================

function GetNCharacters alias "GetNCharacters" (byval buf as GAP_BUFFER_T Ptr, b() As Integer, StartIndex As integer, MaxChars As integer) As integer
  Dim n As integer
  n = Length - StartIndex
  If StartIndex >= 0 And n > 0 And MaxChars > 0 Then
    If n > MaxChars Then
      n = MaxChars
    End If
    ReDim b(0 To n - 1)
    
    If StartIndex >= .mGap1 Then
      CopyMemory b(0), m(StartIndex + (.mGap2 - .mGap1)), n * 2
    ElseIf StartIndex < .mGap1 And StartIndex + n <= .mGap1 Then
      CopyMemory b(0), m(StartIndex), n * 2
    Else
      CopyMemory b(0), m(StartIndex), (.mGap1 - StartIndex) * 2
      CopyMemory b(.mGap1 - StartIndex), m(.mGap2), (n - (.mGap1 - StartIndex)) * 2
    End If
  Else
    n = 0
  End If
  GetNCharacters = n
End Function

function getSubText alias "getSubText" (byval buf as GAP_BUFFER_T Ptr, StartIndex As integer, MaxChars As integer) As String
  Dim b() As Integer, n As integer, t As String
  n = GetNCharacters(b(), StartIndex, MaxChars)
  If n > 0 Then
    t = String(n, 0)
    CopyMemoryToPtr StrPtr(t), b(0), n * 2
    SubText = t
  Else
    SubText = ""
  End If
End function

function ScanToCharacter alias "ScanToCharacter" (byval buf as GAP_BUFFER_T Ptr, StartIndex As integer, ch As integer) As integer
  Dim i As integer
  ScanToCharacter = -1
  i = PointerToPtr(StartIndex, True)
  Do While i < .msize
    If m(i) = ch Then
      ScanToCharacter = PtrToPointer(i)
      Exit Do
    End If
    i = i + 1
    If i = .mGap1 Then
      i = .mGap2
    End If
  Loop
End Function
#endif




